/* Nessus Attack Scripting Language
 *
 * Copyright (C) 1999 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include <includes.h>
#include <nasl_raw.h>
#include "function_call.h"
#include "init.h"
#include "sanitize.h"
#include "defines.h"
#include "strutils.h"
#include "pkt_utils.h"
#include "boolean.h"

/*
 * Input : 'function(a:1, b, c:3)'
 *
 * Output : 
 * 	- function
 *	- args :
 *		'a' : 1
 *		''  : b
 *		'c' : 3
 *
 */
static struct arglist * split_function_args(globals, args)
 struct arglist * globals;
 char * args;
{
 struct arglist * ret = emalloc(sizeof(struct arglist));
 struct arglist * fargs = emalloc(sizeof(struct arglist));
 char * copy; 
 char * v, *e;
 char * end;
#ifdef DEBUG 
 printf("split %s\n", args);
#endif 
 copy = strdup(args);
 v = strchr(copy, START_FUNCTION);
 e = my_strchr(copy, START_FUNCTION, END_FUNCTION);
 if(!e){
 	printf("ERROR : no %c in  %s\n", END_FUNCTION, copy);
	return(NULL);
	}
 v[0] = '\0';v++;
 e[0] = '\0';
 
 end = e+1;
 arg_add_value(ret, "atom", ARG_STRING, strlen(copy), strdup(copy));
 /*
  * v points on the first function argument
  */
 while(v && v[0])
 {
  char * w;
  e = v;
  /*
   * We look for the arguments separator, taking care
   * of quoted text.
   */
  while(e && e[0] && e[0]!=FUNCTION_ARGS_SEP){
  	if(e[0]==OPEN_QUOTE)e = strchr(e+1, CLOSE_QUOTE);
	else if(e[0]==START_FUNCTION)e=strchr(e+1, END_FUNCTION);
	if(e && e[0])e++;
	}
  if(e && e[0])e[0]='\0';
  /*
   * v now points on xxxxxxx:xxxxxx or xxxxxx
   */
   w = quoted_strchr(v, FUNCTION_ARGS);
   
   if(w&&!is_function(v))
   {
    /* Nominated arguments */
    w[0] = '\0';
#ifdef DEBUG
    printf("add : %s:%s\n", v, w+1);
#endif    
    arg_add_value(fargs, v, ARG_STRING, strlen(w+1), strdup(w+1));
   }
   else
   {
   arg_add_value(fargs, "no_name", ARG_STRING, strlen(v), strdup(v));
#ifdef DEBUG   
   printf("add : %s\n", v);
#endif   
   }
 if(e)v = e+1;  
 else v = NULL;
 if(v >= end)v = NULL;
 }
 if(end[0])
 {
  if(!strncmp(end, FUNCTION_REPEAT, STRLEN_FUNCTION_REPEAT))
  {
   char * rep = end + STRLEN_FUNCTION_REPEAT;
   char * ed  = my_strchr(end, START_FUNCTION, END_FUNCTION);
   if(ed)ed[0] = '\0';
   arg_add_value(ret, "repeat", ARG_STRING, strlen(rep), strdup(rep));
  }
  else if(!strncmp(end, FUNCTION_PROVIDED, STRLEN_FUNCTION_PROVIDED))
  {
   char * rep = end + STRLEN_FUNCTION_PROVIDED;
   char * ed  = my_strchr(end, START_FUNCTION, END_FUNCTION);
   if(ed)ed[0] = '\0';
   arg_add_value(ret, "provided", ARG_STRING, strlen(rep), strdup(rep));
  }
 }
 arg_add_value(ret, "args", ARG_ARGLIST, -1, fargs); 
 free(copy);
 return(ret);
}
   
    
struct arglist
function_call(globals, args)
 struct arglist * globals;
 char * args;
{
 struct arglist * fdata 	= split_function_args(globals, args);
 struct arglist * functions 	= arg_get_value(globals, "functions");
 callback_func_t func;
 struct arglist * arguments;
 int repeat = 1;
 char * provided;
 struct arglist rt;
 struct arglist * a;
 struct arglist * delete 	= emalloc(sizeof(struct arglist));
 if(!fdata){
   rt.type = PKT_ERROR(ERR_FUNC);
   return(rt);
   }
 func 	= arg_get_value(functions, arg_get_value(fdata, "atom"));
 arguments = arg_get_value(fdata, "args");
 provided = arg_get_value(fdata, "provided");

 bzero(&rt, sizeof(struct arglist));
 if(!func)
 {
  printf("Error ! Function \"%s\" does not exist\n", (char*)
  					arg_get_value(fdata, "atom"));
  rt.type = PKT_ERROR(ERR_FUNC|ERR_EXIST);
  return(rt);
 }

if(arg_get_type(fdata, "repeat")>=0)
 {
  char * rp = arg_get_value(fdata, "repeat");
  int len = arg_get_length(fdata, "repeat");
  if(rp[len-1]==';')rp[len-1]='\0';
  rt = sanitize_variable(globals, rp);
  if(rt.type & VAR_STR)repeat = atoi(rt.value);
  else {
   printf("Error : %s is not a good repeat variable\n", rp);
   rt.type = PKT_ERROR(ERR_VAR|ERR_TYPE);
   return(rt);
   }
  if(rt.type & VAR_DELETE)free(rt.value);
 }else repeat = 1;
 
 /*
  * sanitize the arguments
  */
 a = arguments;
 while(a && a->next)
 { 
 if(strcmp(a->name, "no_name"))
  {
  rt = sanitize_variable(globals, a->value);
  if(rt.type){
	a->value = rt.value;
	a->length = rt.length;
	if(rt.type & VAR_DELETE)arg_add_value(delete, "delete", ARG_PTR,
						rt.length, 
						rt.value);				
	}
  else arg_add_value(delete, "delete", ARG_PTR, -1, a->value);	
  }
  else arg_add_value(delete, "delete", ARG_PTR, -1, a->value);
  
  a = a->next;
 }

 while(repeat--)
 {
  int err;
  err = evaluate_boolean(globals, provided);
  if(err < 0){
  	rt.type = err;
	return(rt);
	}
  if(err)rt = (*func)(globals, arguments);
 }
 arg_free(arguments);
 while(fdata && fdata->next)
 {
  struct arglist * t = fdata->next;
  free(fdata->name);
  if(fdata->type != ARG_ARGLIST)free(fdata->value);
  free(fdata);
  fdata = t;
 }
 if(fdata)free(fdata);
 arg_free_all(delete);
 if(!rt.type) 
 {
  char * zero = "0"; 
  rt.type = VAR_STR|STR_ALL_DIGIT;
  rt.value = strdup(zero);
  rt.length = 1;
 }
 return(rt);
}
 
 
