/* Nessus Attack Scripting Language
 *
 * Copyright (C) 1999 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include <includes.h>
#include <nasl_raw.h>
#include <nessus/pcap.h>
#include "ip.h"
#include "udp.h"
#include "tcp.h"
#include "igmp.h"
#include "icmp.h"
#include "send_packet.h"
#include "init.h"
#include "prompt.h"
#include "pkt_utils.h"
#include "pcap_next.h"
#include "sanitize.h"
#include "defines.h"
#include "nasl_rpc.h"
#ifdef NESSUS_EXTENSIONS
#include "nessus_extensions.h"
#endif


struct interface_info {
    char name[64];
    struct in_addr addr;
};

/* 
 * defined in libnessus
 */
extern struct interface_info* getinterfaces(int *);
struct arglist * init_nasl();


static void init_const(struct arglist *, struct arglist *);
struct arglist *
init_nasl()
{
 char * functions[] = { "forge_ip_packet",
   			"insert_ip_options",
 			"set_ip_elements",
			"get_ip_element",
 			"forge_tcp_packet",
			"get_tcp_element",
			"set_tcp_elements",
			"forge_udp_packet",
			"get_udp_element",
			"set_udp_elements",
			"forge_igmp_packet",
			"forge_icmp_packet",
			"get_icmp_element",
  			"dump_ip_packet",
			"dump_tcp_packet",
			"dump_udp_packet",
 			"send_packet",
			"prompt",
			"crap",
			"display",
			"this_host",
			"exit",
			"string",
			"raw_string",
			"strlen",
			"tolower",
			"strstr",
			"pcap_next",
			"usleep",
			"sleep",
			"rand",
			"free_pkt",
			"getrpcport",
			"ord",
			"strtoint",
			"rawtostr",
			"hex",
			"shift_left",
			"shift_right",
			"asctime",
#ifdef NESSUS_EXTENSIONS
			"script_name",
			"script_timeout", 
			"script_description",
			"script_copyright",
			"script_summary",
			"script_category",
			"script_family",
			"script_dependencie",
			"script_require_keys",
			"script_require_ports",
			"script_exclude_keys",
			"script_add_preference",
			"script_get_preference",
			"get_host_name",
			"get_host_ip",
			"get_host_open_port",
			"get_port_state",
			"open_sock_tcp",
			"open_sock_udp",
			"close",
			"ftp_log_in",
			"ftp_get_pasv_port",
			"is_cgi_installed",
			"set_kb_item",
			"get_kb_item",
			"security_hole",
			"security_warning", 
			"recv",
			"recv_line",
			"send",
			"telnet_init",
			"tcp_ping",
			"scanner_add_port",
			"scanner_status",
			"scanner_get_port",
			"islocalhost",
			
#endif
			NULL};
 callback_func_t cbacks[] = {forge_ip_packet,
   			     insert_ip_options,
 			     set_ip_elements,
			     get_ip_element,
 			     forge_tcp_packet,
			     get_tcp_element,
			     set_tcp_elements,
			     forge_udp_packet,
			     get_udp_element,
			     set_udp_elements,
			     forge_igmp_packet,
			     forge_icmp_packet,
			     get_icmp_element,
 			     dump_ip_packet,
			     dump_tcp_packet,
			     dump_udp_packet,
 			     send_packet,
			     pkt_prompt,
			     crap,
			     display,
			     this_host,
			     script_exit,
			     string,
			     raw_string,
			     pkt_strlen,
			     nasl_tolower,
			     pkt_strstr,
			     pkt_pcap_next,
			     pkt_usleep,
			     pkt_sleep,
			     pkt_rand,
			     free_pkt,
			     nasl_getrpcport,
			     pkt_ord,
			     strtoint,
			     rawtostr,
			     pkt_hex,
			     shift_left,
			     shift_right,
			     asctime,
#ifdef NESSUS_EXTENSIONS
			     script_name,
			     script_timeout,	
			     script_description,
			     script_copyright,
			     script_summary,
			     script_category,
			     script_family,
			     script_dependencie,
			     script_require_keys,
			     script_require_ports,
			     script_exclude_keys,
			     script_add_preference,
			     script_get_preference,
			     get_hostname,
			     get_host_ip,
			     get_host_open_port,
			     get_port_state,
			     pkt_open_sock_tcp,
			     pkt_open_sock_udp,
			     pkt_close,
			     pkt_ftp_log_in,
			     pkt_ftp_get_pasv_address,
			     pkt_is_cgi_installed,
			     set_kb_item,
			     get_kb_item,
			     security_hole,
			     security_warning,
			     pkt_recv,
			     pkt_recv_line,
			     pkt_send,
			     telnet_init,
			     tcp_ping,
			     nasl_scanner_add_port,
			     nasl_scanner_status,
			     nasl_scanner_get_port,
			     nasl_islocalhost,
#endif			    	
			    NULL};			
 struct arglist * ret;
 struct arglist * funcs;
 struct arglist * vars;
 struct arglist * vars_types;
 struct arglist * udp_sockets;
 int i;
 int soc;
 pcap_t * pcap;
 char * errbuf = emalloc(PCAP_ERRBUF_SIZE);
 struct arglist * pcaps = emalloc(sizeof(struct arglist));
 struct interface_info * mydevs;
 int numdevs;
 
 /*
  * Initialize the libpcap. Rather than ask the user which
  * interface to use, we open a pcap on EVERY interface, and
  * we will determine which to use at run time
  */
 mydevs = (struct interface_info*)getinterfaces(&numdevs);
 for(i=0;i<numdevs;i++)
 { 
  if(!arg_get_value(pcaps, mydevs[i].name))
  {
   pcap = NULL;
#ifndef NESSUS_EXTENSIONS
   /*
    * Yup, the interface is in promisc mode
    */
   pcap = pcap_open_live(mydevs[i].name, 1500, 1, 1, errbuf);
#else
   pcap = pcap_open_live(mydevs[i].name, 1500, 0,1,errbuf);
#endif  
   
   /*
    * Most hosts have interfaces that are down
    */
   if(pcap)arg_add_value(pcaps, strdup(mydevs[i].name), ARG_PTR, -1, pcap);
  }
 }
 efree(&errbuf);
 
 /*
  * Now, open our raw socket
  */
 soc = socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
 if(soc < 0)
 {
  perror("socket ");
  printf("*** Could not open a raw socket\n");
  /* exit(0); */
 }
 
 
 i = 1;
 if((setsockopt(soc, IPPROTO_IP, IP_HDRINCL, (char*)&i, sizeof(i)))<0)
	perror("setsockopt ");


 ret = emalloc(sizeof(struct arglist));
 arg_add_value(ret, "socket", ARG_INT, sizeof(soc), (void*)soc);
 arg_add_value(ret, "pcaps", ARG_PTR, -1, (void*)pcaps);
 funcs = emalloc(sizeof(struct arglist));
 arg_add_value(ret, "functions", ARG_ARGLIST, -1, funcs);
 for(i=0;functions[i];i++)
  arg_add_value(funcs, functions[i], ARG_PTR, -1, cbacks[i]);
 vars = emalloc(sizeof(struct arglist));
 vars_types = emalloc(sizeof(struct arglist));
 init_const(vars, vars_types);
 udp_sockets = emalloc(sizeof(struct arglist ));
 arg_add_value(vars_types, "__udp_sockets", ARG_ARGLIST, -1, udp_sockets);
 arg_add_value(ret, "variables", ARG_ARGLIST, -1, vars); 
 arg_add_value(ret, "variables_types", ARG_ARGLIST, -1, vars_types);
 return(ret);
} 


/*
 * The constants are (in fact) pre-defined variables. 
 */
static void
init_const(vars, types)
 struct arglist * vars, * types;
{
 char * t = emalloc(20);
 sprintf(t, "%d", IPPROTO_TCP);
 arg_add_value(vars, "IPPROTO_TCP", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"IPPROTO_TCP", ARG_INT, sizeof(int),
 		(void*)((VAR_STR|VAR_CONST|STR_ALL_DIGIT)));
 
 sprintf(t, "%d", IPPROTO_UDP);
 arg_add_value(vars, "IPPROTO_UDP", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"IPPROTO_UDP", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 
 sprintf(t, "%d", IPPROTO_ICMP);
 arg_add_value(vars, "IPPROTO_ICMP", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"IPPROTO_ICMP", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 
 sprintf(t, "%d", IPPROTO_IP);
 arg_add_value(vars, "IPPROTO_IP", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"IPPROTO_IP", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 
 sprintf(t, "%d", IPPROTO_IGMP);
 arg_add_value(vars, "IPPROTO_IGMP", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"IPPROTO_IGMP", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 
 sprintf(t, "%d", TH_FIN);
 arg_add_value(vars, "TH_FIN", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"TH_FIN", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 sprintf(t, "%d", TH_SYN);
 arg_add_value(vars, "TH_SYN", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"TH_SYN", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 sprintf(t, "%d", TH_RST);
 arg_add_value(vars, "TH_RST", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"TH_RST", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 sprintf(t, "%d", TH_PUSH);
 arg_add_value(vars, "TH_PUSH", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"TH_PUSH", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 sprintf(t, "%d", TH_ACK);
 arg_add_value(vars, "TH_ACK", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"TH_ACK", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 sprintf(t, "%d", TH_URG);
 arg_add_value(vars, "TH_URG", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"TH_URG", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 sprintf(t, "%d", TH_FLAGS);
 arg_add_value(vars, "TH_FLAGS", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"TH_FLAGS", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 sprintf(t, "%d", IP_RF);
 arg_add_value(vars, "IP_RF", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"IP_RF", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 sprintf(t, "%d", IP_DF);
 arg_add_value(vars, "IP_DF", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"IP_DF", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 sprintf(t, "%d", IP_MF);
 arg_add_value(vars, "IP_MF", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"IP_MF", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 sprintf(t, "%d", IP_OFFMASK);
 arg_add_value(vars, "IP_OFFMASK", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types,"IP_OFFMASK", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 sprintf(t, "1");
 arg_add_value(vars, "TRUE", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types, "TRUE", ARG_INT, sizeof(int), (void*)(VAR_STR|STR_ALL_DIGIT));
 sprintf(t, "0");
 arg_add_value(vars, "FALSE", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types, "FALSE", ARG_INT, sizeof(int),(void*)(VAR_STR|STR_ALL_DIGIT));
 arg_add_value(vars, "NULL", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types, "NULL", ARG_INT, sizeof(int), (void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
 arg_add_value(vars, "pcap_timeout", ARG_STRING, -1, strdup("1000"));
 arg_add_value(types, "pcap_timeout", ARG_INT, sizeof(int),
  (void*)(VAR_STR|STR_ALL_DIGIT));
  
#ifdef NESSUS_EXTENSIONS
 arg_add_value(vars, "ACT_GATHER_INFO", ARG_INT, sizeof(int),
 	(void*)ACT_GATHER_INFO);
 arg_add_value(types, "ACT_GATHER_INFO", ARG_INT, sizeof(int),
 	(void*)(VAR_INT|VAR_CONST));
	
 arg_add_value(vars, "ACT_ATTACK", ARG_INT, sizeof(int),
 	(void*)ACT_ATTACK);
 arg_add_value(types, "ACT_ATTACK", ARG_INT, sizeof(int),
 	(void*)(VAR_INT|VAR_CONST));	

 arg_add_value(vars, "ACT_DENIAL", ARG_INT, sizeof(int),
 	(void*)ACT_DENIAL);
 arg_add_value(types, "ACT_DENIAL", ARG_INT, sizeof(int),
 	(void*)(VAR_INT|VAR_CONST));
	
  arg_add_value(vars, "ACT_SCANNER", ARG_INT, sizeof(int),
 	(void*)ACT_SCANNER);
 arg_add_value(types, "ACT_SCANNER", ARG_INT, sizeof(int),
 	(void*)(VAR_INT|VAR_CONST));
 sprintf(t, "%d", MSG_OOB);
 arg_add_value(vars, "MSG_OOB", ARG_STRING, strlen(t), strdup(t));
 arg_add_value(types, "MSG_OOB", ARG_INT,
 	sizeof(int),(void*)(VAR_STR|VAR_CONST|STR_ALL_DIGIT));
#endif
 efree(&t);
}
