/*
 *          Copyright (c) mjh-EDV Beratung, 1996-1999
 *     mjh-EDV Beratung - 63263 Neu-Isenburg - Rosenstrasse 12
 *          Tel +49 6102 328279 - Fax +49 6102 328278
 *                Email info@mjh.teddy-net.com
 *
 *    Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
 *
 *   $Id: common-stuff.h,v 1.4 1999/11/05 19:20:15 jordan Exp $
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __COMMON_STUFF_H__
#define __COMMON_STUFF_H__

/* ----------------------------------------------------------------------- *
 *                  begin of configurable section                          *
 * ----------------------------------------------------------------------- */

/* All we need here is an independent send/reveive structure provided by
   the operating system. */

#define OS_SEND(fd,buf,len,flg) send (fd, buf, len, flg)
#define OS_RECV(fd,buf,len,flg) recv (fd, buf, len, flg)

/* ----------------------------------------------------------------------- *
 *                    end of configurable section                          *
 * ----------------------------------------------------------------------- */

#ifdef HAVE_CONFIG_H
# include "peks-config.h"
#else
# ifdef _WIN32
#  include "peks-config.w32"
#  include <windows.h>
# endif
#endif

#ifdef HAVE_STDIO_H
# include <stdio.h>
#endif
#ifdef HAVE_UNISTD_H
# include <sys/types.h>
# include <unistd.h>
#else
# ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
# endif
#endif

/* size_t is defined in types.h */
#include "memalloc.h"

#if STDC_HEADERS
# include <string.h>
# include <stdlib.h>
#else
# ifdef HAVE_STRING_H
#  include <string.h>
# endif
# ifdef HAVE_STRINGS_H
#  include <strings.h>
# endif
# ifdef HAVE_MEMORY_H
#  include <memory.h>
# endif
# ifndef HAVE_STRCHR
#  define strchr index
#  define strrchr rindex
# endif
# ifndef _WIN32
  char *strchr (), *strrchr ();
# endif
# ifdef HAVE_BCOPY  
#  ifndef HAVE_MEMCPY
#   define memcpy(d, s, n) bcopy ((s), (d), (n))
#  endif
#  ifndef HAVE_MEMMOVE
#   define HAVE_MEMMOVE
#   define memmove(d, s, n) bcopy ((s), (d), (n))
#  endif
# endif
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif

#include <limits.h>

#ifndef HAVE_STRCASECMP
# ifdef HAVE__STRICMP
#  define strcasecmp(s,t)    _stricmp(s,t)
#  define strncasecmp(s,t,n) _strnicmp(s,t,n)
# else
extern int xstrcasecmp  (const char*,const char*);
extern int xstrncasecmp (const char*,const char*,size_t);
#  define strcasecmp(s,t)    xstrcasecmp(s,t)
#  define strncasecmp(s,t,n) xstrncasecmp(s,t,n)
# endif
#endif

#ifndef HAVE_MEMMOVE
/* redefine, so we do not bother if some prototypes hangs around */
extern void movemem (unsigned char *trg, unsigned char *src, unsigned len);
#define memmove(x,y,z) movemem (x, y, z)
#endif /* HAVE_MEMMOVE */

#ifdef HAVE_32ALIGN
#if SIZEOF_UNSIGNED_INT == 4
  typedef unsigned int u32;
#elif SIZEOF_UNSIGNED_LONG == 4
  typedef unsigned long u32;
#else
# error no typedef for u32
#endif
#endif /* HAVE_32ALIGN */

#ifdef HAVE_64ALIGN
#if SIZEOF_UNSIGNED_LONG == 8
  typedef unsigned long u64;
#elif SIZEOF_UNSIGNED_LONG_LONG == 8
  typedef unsigned long u64;
#else
# error no typedef for u64
#endif
#endif /* HAVE_64ALIGN */

#ifdef HAVE_GETTIMEOFDAY
# ifdef GETTIMEOFDAY_ONE_ARGUMENT
#  define gettimeofday2(x,y) gettimeofday(x)
# else
#  define gettimeofday2(x,y) gettimeofday(x,y)
# endif
# define HAVE_GETTIMEOFDAY2
#else
# ifdef HAVE_TIMEVAL
   extern int gettimeofday2 (struct timeval*, struct timezone*);
#  define HAVE_GETTIMEOFDAY2
# endif
#endif

# ifdef _WIN32
# include <sys/stat.h>
# include  <time.h>

extern time_t __peks_nt_mktime (struct tm *);
extern int    __peks_nt_read  (HANDLE fd, char *buf, unsigned len);
extern HANDLE __peks_nt_open  (const char *, unsigned, unsigned);
extern int    __peks_nt_close (HANDLE fd);
extern FILE * __peks_nt_fopen (const char *file, const char *mode);
extern int    __peks_nt_rename (const char *old, const char *newf);

#  define mktime(x)            __peks_nt_mktime (x)
#  define open(file,mode)      __peks_nt_open (file,mode,0)
#  define open3(file,mode,msk) __peks_nt_open (file,mode,msk)
#  define close(fd)            __peks_nt_close (fd)
#  define read(fd, buf, len)   __peks_nt_read (fd, buf, len) 
#  define fopen(file, mode)    __peks_nt_fopen (file, mode)
#  define rename(old,newf)     __peks_nt_rename (old, newf)

#  define M_HIDDEN       01000 /* creation mask, M$ only */
#  define M_RDONLY       02000 /* M$ emulation, only */
#  define OPEN_ERROR     INVALID_HANDLE_VALUE

#  ifndef O_RDONLY
#   define O_RDONLY            1
#   define O_WRONLY            2
#   define O_RDWR              4 /* 3 would do, as well */
#   define O_CREAT             8
#   define O_TRUNC            16
#   define O_EXCL             32
#   define O_APPEND           64
#  endif

# else
#  define HANDLE              int
#  define OPEN_ERROR          (-1)
#  define M_HIDDEN            0 /* M$ emulation, only */
#  define M_RDONLY            0 /* M$ emulation, only */
#  define open3(f,m,x)        open (f,m,x)
# endif

#endif /* __COMMON_STUFF_H__ */
